/*  26 November 2007. UIAV Venice mobile services workshop. 
    By Nicholas Zambetti
    This code will search for nearby bluetooth devices
    Pressing Ok/Confirm will initiate search/discovery
    Pressing 0 Key will cancel discovery then another press will exit the program
    Tested on a Nokia 3230 Phone only 
*/

import processing.phone.*;         // import phone library to go fullscreen
import processing.bluetooth.*;     // import the bluetooth library

Phone p;                           // named reference to phone instance
Bluetooth bt;                      // named reference to bluetooth instance
PFont font;                        // named reference to font
color white;                       // named reference to colors...
color darkBlue;
color lightBlue;

boolean discovering = false;       // variable to indicate whether or not we are discovering
int deviceCount = 0;               // variable to store the number of devices we've discovered

void setup()                                  // when program starts...
{
  p = new Phone(this);                        // create new phone instance
  p.fullscreen();                             // go fullscreen
  bt = new Bluetooth(this);                   // create new bluetooth instance
  white = color(255, 255, 255);               // specify colors...
  darkBlue = color(0, 0, 50);
  lightBlue = color(0, 0, 100);
  font = loadFont("Verdana-10.mvlw", white);  // load font
  textFont(font);                             // set the font we've loaded as the default
  background(lightBlue);                      // draw the background (clear the screen)
}

void draw()                                                          // repeatedly (according to framerate)...
{
  fill(darkBlue);                                                    // change the fill color for future drawing routines
  rect(0, 0, width, 14);                                             // draw a rectangle for the bar at the top

  if(discovering)                                                    // if we are discovering...
  {
    if(1 <= deviceCount)                                             // if we've found at least one device
    {
      text("Bluetooth Devices Found: " + nf(deviceCount, 2), 5, 12); // tell the user how many devices we've found
    } 
    else {                                                           // we haven't found any devices yet...
      text("Discovering Bluetooth Devices...", 5, 12);               // tell the user that we've started discovering
      fill(lightBlue);                                               // change the fill color for future drawing routines
      rect(0, 14, width, height - 14);                               // draw a rectangle for the bar at the top
    }
  }
  else{                                                              // we are not discovering...
    text("Press Select To Start...", 5, 12);                         // tell the user how to start discovery
  } 
}

void keyPressed()              // whenever a key is pressed...
{
  if (keyCode == FIRE)         // if we've pressed the center key on the directional pad...
  {
    if(discovering == false)   // if we are not discovering...
    {
      discovering = true;      // indicate that we are discovering
      deviceCount = 0;         // and that we haven't found any devices yet
      bt.discover();           // start discovery
    }
  }
  if(keyCode == '0')           // if we've pressed the '0' key on the numeric keypad...
  {
    if(discovering == true)    // if we are discovering...
    {
      discovering = false;     // indicate that we are no longer discovering
      bt.cancel();             // discontinue discovery
    } else {                   // we are not discovering...
      exit();                  // exit program
    }
  }
}

void destroy()   // whenever exit() is called...
{
  bt.cancel();   // discontinue bluetooth discovery
}

void libraryEvent(Object library, int event, Object data)                     // whenever a library event occurs...
{
  if (library == bt)                                                          // if the bluetooth library sent the event...
  {
    switch (event)                                                            // switch behavior for each type of event...
    {
    case Bluetooth.EVENT_DISCOVER_DEVICE:                                     // in case of the event of a device being discovered
      Device device = (Device) data;                                          // temporarily store the discovered device's information
      text(device.address + ": " + device.name, 5, 26 + (deviceCount * 16));  // tell the user about the device (device name is perhaps still unknown)
      ++deviceCount;                                                          // increment device count
      break;                                                                  // discontinue (break) the handling of this case

    case Bluetooth.EVENT_DISCOVER_DEVICE_COMPLETED:                           // in case of the event of device discovery being completed
      discovering = false;                                                    // indicate that we are no longer discovering
      Device[] devices = (Device[]) data;                                     // store found devices in an array
      fill(lightBlue);                                                        // change the fill color for future drawing routines
      rect(0, 14, width, height - 14);                                        // draw a rectangle for the bar at the top
      for(int i = 0; i < devices.length; ++i) {                               // for each device in the array...
        text(devices[i].name + " " + devices[i].address, 5, 26 + (i * 16));   // tell the user about the device
      }
      break;                                                                  // discontinue (break) the handling of this case
    }
  }
}
